!***********************************************************************
!*****************************  KEK, High Energy Accelerator Research  *
!*****************************  Organization                           *
!** u c p h a n t o m c g v **                                         *
!*****************************     EGS5.0 USER CODE - 18 Jul 2005/1300 *
!***********************************************************************
!***********************************************************************
!                                                                      *
!  PROGRAMMERS:  H. Hirayama                                           *
!                Radiation Science Center                              *
!                Applied Science Laboratory                            *
!                KEK, High Energy Accelerator Research Organization    *
!                1-1, Oho, Tsukuba, Ibaraki, 305-0801                  *
!                Japan                                                 *
!                                                                      *
!                E-mail:      hideo.hirayama@kek.jp                    *
!                Telephone:   +81-29-864-5489                          *
!                Fax:         +81-29-864-1993                          *
!                                                                      *
!***********************************************************************
!***********************************************************************
! The ucphantomcgv.f User Code requires a cg-input file only           *
! (e.g., ucphantomcgv.data).                                           *
! The following shows the geometry for ucphantomcgv.data.              *
! Cg-data can be checked by CGview.                                    *
! This user code corresponds to ucphantomcgp.mor for egs4.             *
! Use Ranlux random number generator.                                  *
!***********************************************************************
!                                                                      *
!             --------------------------                               *
!             cg Geometry (ucphantomcgv)                               *
!             ---------------------------                              *
!                                                                      *
!                                                                      *
!                X                                                     *
!                ^                                                     *
!                |                                                     *
!           +----+----+----+----   -+---+----+----+--- 20.0            *
!           |                                     |                    *
!           |     Outer vacuum region             |                    *
!           +    +----+----+----   -+---+----+    +--- 15.0            *
!           |    |    |                 |    |    |                    *
!           |    +    +  Water (H2O)    |    |    |                    *
!           |    |    |                 |    |    |                    *
!           +    +    +---+---+-   -+---+    +    +--   0.5            *
!           |    |    |   |   |     |   |    |    |                    *
!           |    |Air |H2O|H2O|     |H2O|Air |    |                    *
! X-ray     |    |    |   |   |     |   |    |    |                    *
!      ==========>+---+---+---+-   -+---+----+----+--> Z               *
! photons -20   -5.0  0  1.0 2.0  19.0 20.0     40.0                   *
!                                          25.0                        *
!                                                                      *
!***********************************************************************
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
!
!-----------------------------------------------------------------------
!------------------------------- main code -----------------------------
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! Step 1: Initialization
!-----------------------------------------------------------------------

      implicit none

!     ------------
!     EGS5 COMMONs
!     ------------
      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_bounds.f'
      include 'include/egs5_edge.f'
      include 'include/egs5_elecin.f'
      include 'include/egs5_media.f'
      include 'include/egs5_misc.f'
      include 'include/egs5_stack.f'
      include 'include/egs5_thresh.f'
      include 'include/egs5_uphiot.f'
      include 'include/egs5_useful.f'
      include 'include/egs5_usersc.f'
      include 'include/egs5_userxt.f'
      include 'include/randomm.f'

!     ----------------------
!     Auxiliary-code COMMONs
!     ----------------------
      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/edata.f'
      include 'auxcommons/etaly1.f'
      include 'auxcommons/instuf.f'
      include 'auxcommons/lines.f'
      include 'auxcommons/nfac.f'
      include 'auxcommons/watch.f'

      include 'auxcommons/etaly2.f'      !  Added SJW for energy balance

!     ------------------
!     cg related COMMONs
!     ------------------
      include 'auxcommons/geom_common.f' ! geom-common file
      integer irinn

      common/totals/                             ! Variables to score
     * depe(20),faexp,fexps,imode,ndet
      real*8 depe,faexp,fexps
      integer imode,ndet

!**** real*8                                                 ! Arguments
      real*8 etot,totke
      integer ins

!**** real*8                                           ! Local variables
      real*8
     * area,availke,depthl,depths,dis,disair,ei0,ekin,elow,eup,
     * phai0,phai,radma2,sinth,sposi,tnum,vol,w0,wimin,wtin,wtsum,
     * xhbeam,xpf,yhbeam,ypf

      real*8 bsfa,bsferr,faexps,faexp2s,faexrr,fexpss,fexps2s,fexerr,
     *       faexpa,fexpsa

      real*8
     * depeh(20),depeh2(20),dose(20),dose2(20),doseun(20),ebint(201),
     * nofebin(1),deltae(1),sspec(1,201),ecdft(201),saspec(201)

      real
     * tarray(2),tt,tt0,tt1,cputime

      integer
     * i,ii,ibatch,icases,idin,ie,ifti,ifto,imed,ireg,isam,
     * ixtype,j,k,kdet,nlist,nnn,nsebin

      character*24 medarr(2)

!     ----------
!     Open files
!     ----------
!-----------------------------------------------------------------
!     Units 7-26 are used in pegs and closed.  It is better not
!     to use as output file. If they are used must be re-open after
!     call pegs5.   Unit for pict must be 39.
!-----------------------------------------------------------------

      open(unit= 1,file='egs5job.out',status='unknown')
      open(unit= 2,file='xray.dat',status='old')  ! Data of source x-ray
      open(UNIT= 4,FILE='egs5job.inp',STATUS='old')
      open(unit=39,FILE='egs5job.pic',STATUS='unknown')

!     ====================
      call counters_out(0)
!     ====================

!-----------------------------------------------------------------------
! Step 2: pegs5-call
!-----------------------------------------------------------------------
!     ==============
      call block_set                 ! Initialize some general variables
!     ==============

!     ---------------------------------
!     define media before calling PEGS5
!     ---------------------------------

      nmed=2
      medarr(1)='WATER                   '
      medarr(2)='AIR-AT-NTP              '


      do j=1,nmed
        do i=1,24
          media(i,j)=medarr(j)(i:i)
        end do
      end do  

      chard(1) = 1.0d0       !  optional, but recommended to invoke
      chard(2) = 1.0d0       !  automatic step-size control

!     ------------------------------
!     Run PEGS5 before calling HATCH
!     ------------------------------
      write(6,*) ' PEGS5-call comes next'

!     ==========
      call pegs5
!     ==========

!-----------------------------------------------------------------------
! Step 3: Pre-hatch-call-initialization
!-----------------------------------------------------------------------
!-----------------------------------------------
!     Define pict data mode.
!-----------------------------------------------
!     npreci 1: for PICT32
!            2: for CGview
      npreci=2
!-----------------------------------------------
!     Initialize cg related parameter 
!-----------------------------------------------
      itbody=0
      irppin=0
      isphin=0
      irccin=0
      itorin=0
      itrcin=0
      izonin=0
      itverr=0
      igmmax=0
      ifti = 4     ! Input unit number for cg-data
      ifto = 39    ! Output unit number for PICT

      write(39,100)
100   FORMAT('CSTA')
      call geomgt(ifti,ifto)
      write(39,110)
110   FORMAT('CEND')

!--------------------------------
!     Get nreg from cg input data
!--------------------------------
      nreg=izonin
      if (nreg.gt.mxreg) then
        write(1,120) nreg,mxreg 
120     FORMAT(' NREG(=',I12,') must be less than MXREG(=',I12,')' /
     *  ' You must change MXREG in include/egs5_h.f.')
        stop
      end if

!   Set medium index for each region
!     Vacuum region
      med(nreg)=0

!     Air region
      med(1)=2
      med(nreg-1)=2

!     Water region

      do i=2,nreg-2
        iphter(i) = 1    ! Switches for PE-angle sampling
        iedgfl(i) = 1    ! K & L-edge fluorescence
        iauger(i) = 0    ! K & L-Auger
        iraylr(i) = 1    ! Rayleigh scattering
        lpolar(i) = 0    ! Linearly-polarized photon scattering
        incohr(i) = 0    ! S/Z rejection
        iprofr(i) = 0    ! Doppler broadening
        impacr(i) = 0    ! Electron impact ionization
        med(i)=1         !Water phantom region
      end do

!     --------------------------------
!     Set parameter estepe and estepe2
!     --------------------------------
      estepe=0.10
      estepe2=0.20
      write(6,130) estepe, estepe2
130   FORMAT(1X,'ESTEPE at EKMAX: ',F10.5,' (estepe)',
     *       /,1X,'ESTEPE at ECUT:  ',F10.5,' (estepe2)')

!     --------------------------------------------------------
!     Random number seeds.  Must be defined before call hatch
!     or defaults will be used.  inseed (1- 2^31)
!     --------------------------------------------------------
      luxlev = 1
      inseed=1
      write(6,140) inseed
140   FORMAT(/,' inseed=',I12,5X,
     *         ' (seed for generating unique sequences of Ranlux)')

!     =============
      call rluxinit  ! Initialize the Ranlux random-number generator
!     =============

!-----------------------------------------------------------------------
! Step 4:  Determination-of-incident-particle-parameters
!-----------------------------------------------------------------------
!-----------------------
!     Read spectrum pdf 
!-----------------------
      do i=1,1
        read(2,*) nofebin(i)
        read(2,*) deltae(i) 
        read(2,*) (sspec(i,ie),ie=1,nofebin(i))
      end do

!------------------------
!     Select source type 
!------------------------
150   write(6,160) 
160   FORMAT(' Key in source type. 1:100kV')
      read(5,*) ixtype
      if (ixtype.eq.0.or.ixtype.gt.1) then
        write(6,170)
170     FORMAT(' IXTYPE must be >0 <= 1.')
        go to 150
      end if

!---------------------------------------
!     Calculate CDF for selected source
!---------------------------------------
      nsebin=nofebin(ixtype)
      tnum=0.D0
      do ie=1,nsebin
        tnum=tnum+sspec(ixtype,ie)
      end do

      nsebin=nsebin+1
      ecdft(1)=0.0
      do ie=2,nsebin
        ecdft(ie)=ecdft(ie-1)+sspec(ixtype,ie-1)/tnum
      end do

!---------------------------
!     Make energy bin table
!---------------------------
      do ie=1,nsebin
        ebint(ie)=(ie-1)*deltae(ixtype)
      end do

!-------------------------------------------------
!     Define source position from phantom surface.
!-------------------------------------------------
      write(6,180)
180   FORMAT(' Key in source position from phantom surface in cm')
      read(5,*) sposi

!------------------------------
!     Source condition redefine
!------------------------------
      iqin=0             ! Incident charge - photons
      ekein=ebint(nsebin)        ! Maximum kinetic energy
      etot=ekein + abs(iqin)*RM
      xin=0.D0    
      yin=0.D0    
      zin=-sposi 
      uin=0.D0    
      vin=0.D0    
      win=1.D0    
      irin=0      ! Source region number is defined from xin and yin.

!----------------------------------------------------
!     Key in half width and height at phantom surface
!----------------------------------------------------
      write(6,190)
190   FORMAT(' Key in half width of beam at phantom surface in cm.')
      read(5,*) xhbeam
      write(6,200)
200   FORMAT(' Key in half height of beam at phantom surface in cm.')
      read(5,*) yhbeam
      radma2=xhbeam*xhbeam+yhbeam*yhbeam
      wimin=sposi/dsqrt(sposi*sposi+radma2)

!-----------------------------------
!     Selection mode form Keyboard.
!-----------------------------------
      write(6,210)
210   FORMAT(' Key in mode. 0:trajectory display, 1:dose calculation')
      read(5,*) imode

!-----------------------------------------------------------------------
! Step 5:   hatch-call
!-----------------------------------------------------------------------
! Define posible maximum total energy of electron before hatch
      emaxe = ekein + RM         ! photon

      write(6,220)
220   format(/' Start ucphantomcgv'/
     *        ' Call hatch to get cross-section data')

!     ------------------------------
!     Open files (before HATCH call)
!     ------------------------------
      open(UNIT=KMPI,FILE='pgs5job.pegs5dat',STATUS='old')
      open(UNIT=KMPO,FILE='egs5job.dummy',STATUS='unknown')

      write(6,230)
230   FORMAT(/,' HATCH-call comes next',/)

!     ==========
      call hatch
!     ==========

!     ------------------------------
!     Close files (after HATCH call)
!     ------------------------------
      close(UNIT=KMPI)
      close(UNIT=KMPO)

! ----------------------------------------------------------
! Print various data associated with each media (not region)
! ----------------------------------------------------------
      write(1,240)
240   FORMAT(/,' Quantities associated with each MEDIA:')
      do j=1,nmed
        write(1,250) (media(i,j),i=1,24)
250     FORMAT(/,1X,24A1)
        write(1,260) rhom(j),rlcm(j)
260     FORMAT(5X,' rho=',G15.7,' g/cu.cm     rlc=',G15.7,' cm')
        write(1,270) ae(j),ue(j)
270     FORMAT(5X,' ae=',G15.7,' MeV    ue=',G15.7,' MeV')
        write(1,280) ap(j),up(j)
280     FORMAT(5X,' ap=',G15.7,' MeV    up=',G15.7,' MeV',/)
      end do

      write(1,290)
290   FORMAT(/' Information of medium and cut-off for each region')
      do i=1,nreg
        if (med(i).eq.0) then
          write(1,300) i
300       FORMAT(' Medium(region:',I5,')= Vacuum')
        else
          write(1,310) i,(media(ii,med(i)),ii=1,24),
     *                 ecut(i),pcut(i),rhor(i)
310       FORMAT(' Medium(region:',I5,
     *           ')=',24A1,/5X,'ECUT=',G10.5,' MeV, PCUT=',
     *           G10.5, ' MeV, density=',F10.3)
        end if
      end do

        
      write(39,320)
320   FORMAT('MSTA')
      write(39,330) nreg
330   FORMAT(I4)
      write(39,340) (med(i),i=1,nreg)
340   FORMAT(15I4)
      write(39,350)
350   FORMAT('MEND')

!-----------------------------------------------------------------------
! Step 6:  Initialization-for-howfar
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
! Step 7:  Initialization-for-ausgab
!-----------------------------------------------------------------------

      ncount = 0
      ilines = 0
      nwrite = 10
      nlines = 25
      idin = -1
      totke = 0.
      wtsum = 0.

!     =========================
      call ecnsv1(0,nreg,totke)
      call ntally(0,nreg)
!     =========================

!---------------------
!     Clear variables
!---------------------
      do nnn=1,20
        depe(nnn)=0.D0
        depeh(nnn)=0.D0
        depeh2(nnn)=0.D0
      end do  

      faexp=0.D0
      faexps=0.D0
      faexp2s=0.D0
      fexps=0.D0
      fexpss=0.D0
      fexps2s=0.D0
      
      do i=1,201
        saspec(i)=0.D0
      end do

      ibatch=0

!-----------------------------
!     Detector number to score
!-----------------------------
      write(6,360) nreg-4
360   format(' Key in number of dose calculation region.(<=',I5,')')
      read(5,*) ndet

      write(1,370)
370   FORMAT(//,' Energy/Coordinates/Direction cosines/etc.',/,
     *        6X,'e',16X,'x',14X,'y',14X,'z'/
     *        1X,'u',14X,'v',14X,'w',9X,'iq',4X,'ir',3X,'iarg',/)

!--------------------------
!     Key in history number
!--------------------------
380   write(6,390)
390   FORMAT(' Key in number of cases (0 means end of calculation.)')
      read(5,*) ncases
      if (ncases.eq.0) go to 570

      ibatch=ibatch+1

      close(39,status='keep')
      open(39,file='egs5job.pic',access='append')
      write(39,400) ibatch
400   FORMAT('0',I5)

      tt=etime(tarray)
      tt0=tarray(1)

!-----------------------------------------------------------------------
! Step 8:  Shower-call
!-----------------------------------------------------------------------

                                             ! -------------------------
      do j=1,ncases                          ! Start of CALL SHOWER loop
                                             ! -------------------------
        icases=j

!--------------------------------------
!       Determine direction (isotropic)
!--------------------------------------
410     call randomset(w0)
        win=w0*(1.0-wimin)+wimin
        call randomset(phai0)
        phai=pi*(2.0*phai0-1.0)
        sinth=dsqrt(1.D0-win*win)
        uin=dcos(phai)*sinth
        vin=dsin(phai)*sinth
        dis=sposi/win
        xpf=dis*uin
        ypf=dis*vin
        if (dabs(xpf).gt.xhbeam.or.dabs(ypf).gt.yhbeam) go to 410
        if (sposi.gt.5.0) then
          disair=(sposi-5.0)/win
          xin=disair*uin
          yin=disair*vin
          zin=-5.D0
        else
          xin=0.D0
          yin=0.D0
          zin=-sposi
        end if

!-----------------------------------------
!     Get source region from cg input data
!-----------------------------------------
!
          if(irin.le.0.or.irin.gt.nreg) then
            call srzone(xin,yin,zin,iqin+2,0,irinn)
            call rstnxt(iqin+2,0,irinn)
          else
            irinn=irin
          end if

!       ----------------------
!       Select incident energy
!       ----------------------
        eparte = 0.d0                   ! Initialize some energy-balance
        epartd = 0.d0                   !      tallying parameters (SJW)

        call randomset(ei0)
        do ie=2,nsebin
          if (ei0.lt.ecdft(ie)) then
            go to 420
          end if
        end do

420     if (ie.gt.nsebin) then
          ie=nsebin
        end if
        saspec(ie)=saspec(ie)+1.D0
        if (ecdft(ie).eq.ecdft(ie-1)) then
          ekin=ebint(ie-1)
        else
          ekin=ebint(ie-1)+(ei0-ecdft(ie-1))*(ebint(ie)-ebint(ie-1))/
     *         (ecdft(ie)-ecdft(ie-1))
        end if
        wtin = 1.0

        wtsum = wtsum + wtin               ! Keep running sum of weights
        etot = ekin + iabs(iqin)*RM        ! Incident total energy (MeV)
        availke = etot + iqin*RM        ! Available K.E. (MeV) in system
        totke = totke + availke                 ! Keep running sum of KE

        latchi=0

!       ---------------------------------------------------
!       Print first NWRITE or NLINES, whichever comes first
!       ---------------------------------------------------
        if (ncount .le. nwrite .and. ilines .le. nlines) then
          ilines = ilines + 1
          write(6,430) etot,xin,yin,zin,uin,vin,win,iqin,irinn,idin
430       FORMAT(4G15.7/3G15.7,3I5)
        end if

!       =========================================================
        call shower (iqin,etot,xin,yin,zin,uin,vin,win,irinn,wtin)
!       =========================================================

!       Added for energy balance tests (SJW)
        if(DABS(eparte + epartd - ekin)/ekin .gt. 1.d-10) then
          write(*,440) icases, eparte, epartd
440       FORMAT('Error on # ',I6,' Escape = ',F9.5,' Deposit = ',F9.5)
        endif

!----------------------------------
!      Sum variable and its square.
!----------------------------------

        do kdet=1,ndet
          depeh(kdet)=depeh(kdet)+depe(kdet)
          depeh2(kdet)=depeh2(kdet)+depe(kdet)*depe(kdet)
          depe(kdet)=0.0
        end do

        faexps=faexps+faexp
        faexp2s=faexp2s+faexp*faexp
        faexp=0.0
        fexpss=fexpss+fexps
        fexps2s=fexps2s+fexps*fexps
        fexps=0.0

        ncount = ncount + 1         ! Count total number of actual cases

                                               ! -----------------------
      end do                                   ! End of CALL SHOWER loop
                                               ! -----------------------

      tt=etime(tarray)
      tt1=tarray(1)
      cputime=tt1-tt0
      if (imode.ne.0) then
        write(1,450) cputime
450     format(' Elapsed Time (sec)=',G15.5)
      end if


!-----------------------------------------------------------------------
! Step 9:  Output-of-results
!-----------------------------------------------------------------------
!     ---------------------
!     Write out the results
!     ---------------------
      write(1,460) ncount,ncases,totke,totke/ncount
460   FORMAT(/,' Ncount=',I10,' (actual cases run)',/,
     *       ' Ncases=',I10,' (number of cases requested)',/,
     *       ' TotKE =',G15.5,' (total KE (MeV) in run)'/
     *       ' Average Kinetic enegy =',G15.5,'MeV'/)

      if (totke .le. 0.D0) then
        write(6,470) totke,availke,ncount
470     FORMAT(//,' Stopped in MAIN with TotKE=',G15.5,/,
     *         ' AvailKE=',G15.5, /,' Ncount=',I10)
        stop
      end if

!-----------------------------
!     Sampled source spectrum
!-----------------------------
      if (imode.ne.0) then
        do ie=2,nsebin
          saspec(ie)=saspec(ie)/float(ncases) 
        end do

        write(1,480)
480     FORMAT(/' Comparison between sampled spectrum and original data'
     *  /23X,'   Sampled    Probability',25X,'   Sampled    Probability'
     *  )
        do ie=2,nsebin,2
          if(ie.eq.nsebin) then
            write(1,490) ebint(ie),saspec(ie),ecdft(ie)-ecdft(ie-1)
490         FORMAT(1X,G9.3,' MeV(upper)-- ',2G12.5)
          else 
            write(1,500) ebint(ie),saspec(ie),ecdft(ie)-ecdft(ie-1),
     *      ebint(ie+1), saspec(ie+1),ecdft(ie+1)-ecdft(ie)
500         FORMAT(1X,G9.3,' MeV(upper)-- ',2G12.5,3X, '; ',G9.3,
     *             ' MeV(upper)-- ',2G12.5)
          end if
        end do

        write(1,510) sposi 
510     FORMAT(/' Absorbed energy inside phantom for 100 kV X-ray'/ 
     *    ' Source position ',F10.1,' cm from phantom surface'/ 
     *    ' Within 1cm x 1 cm area after 5 cm air')

        write(1,520) ncases, xhbeam, yhbeam                            
520     FORMAT(1X,I8,' photons normally incident from front side'/ ' Hal
     *f width of beam is ',G15.5,'cm for X and ',G15.5,'cm for Y')      
      end if

!----------------------------------------------
!     Calculate average dose and its deviation
!----------------------------------------------

      area=1.D0*1.D0
      do kdet=1,ndet 
        vol=area*1.D0
        dose(kdet)=depeh(kdet)/ncases
        dose2(kdet)=depeh2(kdet)/ncases
        doseun(kdet)=dsqrt((dose2(kdet)-dose(kdet)*dose(kdet))/ncases)
        dose(kdet)=dose(kdet)*1.602E-10/vol
        doseun(kdet)=doseun(kdet)*1.602E-10/vol
        depths=kdet-1.0
        depthl=kdet
        write(6,530)depths,depthl,(media(ii,med(kdet+1)),ii=1,24),
     *  rhor(kdet+1),dose(kdet),doseun(kdet)
530     FORMAT(' At ',F4.1,'--',F4.1,'cm (',24A1,',rho:',F8.4,')=',
     *  G13.5,'+-',G13.5,'Gy/incident')
        write(1,530) depths,depthl,(media(ii,med(kdet+1)),ii=1,24),
     *  rhor(kdet+1),dose(kdet),doseun(kdet)
      end do
        

!-------------------------------------------------
!     Calculate average exposure and its deviation
!-------------------------------------------------

      faexpa=faexps/ncases
      faexp2s=faexp2s/ncases
      faexrr=dsqrt((faexp2s-faexpa*faexpa)/ncases) 
      faexpa=faexpa*1.6E-10/area
      faexrr=faexrr*1.6E-10/area
      fexpsa=fexpss/ncases
      fexps2s=fexps2s/ncases
      fexerr=dsqrt((fexps2s-fexpsa*fexpsa)/ncases) 
      fexpsa=fexpsa*1.6E-10/area
      fexerr=fexerr*1.6E-10/area
      if (faexpa.gt.0.0) then
        bsfa=fexpsa/faexpa
        bsferr=bsfa*dsqrt((faexrr/faexpa)**2.+(fexerr/fexpsa)**2.)
        write(6,540) faexpa,faexrr,fexpsa,fexerr,bsfa,bsferr
        write(1,540) faexpa,faexrr,fexpsa,fexerr,bsfa,bsferr
540     FORMAT(/' Exposure in free air (using mu_en) =', G15.5,'+-',G15.
     *  5,' Gy/incident'/ ' Exposure at phantom surface (using mu_en) ='
     *  , G15.5,'+-',G15.5,'Gy/incident'/ ' Backscattering factor =',G15
     *  .5,'+-',G15.5)
      else
        write(6,550) faexpa,faexrr,fexpsa,fexerr
        write(1,550) faexpa,faexrr,fexpsa,fexerr
550     FORMAT(/' Exposure in free air (using mu_en) =', G15.5,'+-',G15.
     *  5,' Gy/incident'/ ' Exposure at phantom surface (using mu_en) ='
     *  , G15.5,'+-',G15.5,'Gy/incident')
      end if

!------------------------------------
!     Write end of batch information
!------------------------------------
      call plotxyz(99,0,0,0.D0,0.D0,0.D0,0.D0,0,0.D0) 
      write(39,560)
560   FORMAT('9')
      close(UNIT=39,status='keep')
      go to 380

570   if (imode.ne.0) then
!     =============================
      call ecnsv1(nlist,nreg,totke)
!     =============================
      end if

!     ====================
      call counters_out(1)
!     ====================

!     -----------
!     Close files
!     -----------
      close(UNIT=1)
      close(UNIT=4)

      stop

      end

!-------------------------last line of main code------------------------

!-------------------------------ausgab.f--------------------------------
! Version:   030831-1300
! Reference: SLAC-265 (p.19-20, Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12

! ----------------------------------------------------------------------
! Required subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
! A simple AUSGAB to:
!
!   1) Score energy deposition
!   2) Print out stack information
!   3) Print out particle transport information (if switch is turned on)
!

! ----------------------------------------------------------------------

      subroutine ausgab(iarg)

      implicit none

      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_epcont.f'    ! COMMONs required by EGS5 code
      include 'include/egs5_media.f'
      include 'include/egs5_misc.f'
      include 'include/egs5_stack.f'
      include 'include/egs5_useful.f'

      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/etaly1.f'        ! Auxiliary-code COMMONs
      include 'auxcommons/lines.f'
      include 'auxcommons/ntaly1.f'
      include 'auxcommons/watch.f'

      include 'auxcommons/etaly2.f'      !  Added SJW for energy balance

      common/totals/                             ! Variables to score
     * depe(20),faexp,fexps,imode,ndet
      real*8 depe,faexp,fexps
      integer imode,ndet

      integer                                                ! Arguments
     * iarg

      real*8                                           ! Local variables
     * cmod,dcon,edepwt,encoea,esing

      integer idet,ie,iql,irl

!     -----------------------------------------------------------------
!     Print out particle transport information (if switch is turned on)
!     -----------------------------------------------------------------
!                        ========================
      if (iwatch .gt. 0) call swatch(iarg,iwatch)
!                        ========================

!  ---------------------------------
!  Keep track of how deep stack gets
!  ---------------------------------
      if (np.gt.MXSTACK) then
        write(6,100) np,MXSTACK
100     FORMAT(//' In AUSGAB, np=',I3,' >= maximum stack',
     *       ' allowed which is',I3/1X,79('*')//)
        stop
      end if

!     ------------------------
!     Set some local variables
!     ------------------------
      irl = ir(np)
      iql = iq(np)
      edepwt = edep*wt(np)

!     -----------------------------------------------------------
!     Keep track of energy deposition (for conservation purposes)
!     -----------------------------------------------------------
      if (iarg .lt. 5) then
        esum(iql+2,irl,iarg+1) = esum(iql+2,irl,iarg+1) + edepwt

!  added SJW for particle by particle energy balance
        if(irl.eq.nreg) then
          eparte = eparte + edepwt
        else 
          epartd = epartd + edepwt
        endif
      end if

!-------------------------------------------------
!     Score data ate detector region (region 2-21)
!--------------------------------------------------
      if (irl.ge.2.and.irl.le.nreg-3) then
        idet=irl-1
        if(idet.ge.1.and.idet.le.ndet) then
          depe(idet)=depe(idet)+edepwt/rhor(irl)
        end if
      end if

!--------------------------------
!     Check cross phantom surface
!--------------------------------
      if (abs(irl-irold).eq.1.and.iq(np).eq.0) then
        if((w(np).gt.0.0.and.irl.eq.2).or.(w(np).le.0.0.and.irl.eq.1))
     *  then
          if (dabs(w(np)).ge.0.0349) then
            cmod=dabs(w(np))
          else 
            cmod=0.0175
          end if
          esing=e(np)
          dcon=encoea(esing)            ! PHOTX data
          fexps=fexps+e(np)*dcon*wt(np)/cmod
          if (w(np).lt.0.0) latch(np)=1
          if (w(np).gt.0.0.and.latch(np).eq.0) then
            faexp=faexp+e(np)*dcon*wt(np)/cmod
          end if
        end if
      end if

!     ------------------------------------
!     Output particle information for plot
!     ------------------------------------
      if (imode.eq.0) then
        call plotxyz(iarg,np,iq(np),x(np),y(np),z(np),e(np),ir(np),
     *       wt(np))
      end if

      return

      end

!--------------------------last line of ausgab.f------------------------
!-------------------------------howfar.f--------------------------------
! Version:   050716-1300
! Reference: T. Torii and T. Sugita, "Development of PRESTA-CG 
! Incorporating Combinatorial Geometry in EGS4/PRESTA", JNC TN1410 2002-201,
! Japan Nuclear Cycle Development Institute (2002).
! Improved version is provided by T. Sugita. 7/27/2004
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12

! ----------------------------------------------------------------------
! Required (geometry) subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
! This is a CG-HOWFAR. 
! ----------------------------------------------------------------------

      subroutine howfar
      implicit none
c
      include 'include/egs5_h.f'       ! Main EGS "header" file
      include 'include/egs5_epcont.f'  ! COMMONs required by EGS5 code
      include 'include/egs5_stack.f'
      include 'auxcommons/geom_common.f' ! geom-common file
c
c
      integer i,j,jjj,ir_np,nozone,jty,kno
      integer irnear,irnext,irlold,irlfg,itvlfg,ihitcg
      double precision xidd,yidd,zidd,x_np,y_np,z_np,u_np,v_np,w_np
      double precision tval,tval0,tval00,tval10,tvalmn,delhow
      double precision atvaltmp
      integer iq_np
c
      ir_np = ir(np)
      iq_np = iq(np) + 2
c
      if(ir_np.le.0) then
        write(6,*) 'Stopped in howfar with ir(np) <=0'
        stop
      end if
c
      if(ir_np.gt.izonin) then
        write(6,*) 'Stopped in howfar with ir(np) > izonin'
        stop
      end if
c
      if(ir_np.EQ.izonin) then
        idisc=1
        return
      end if
c
      tval=1.d+30
      itvalm=0
c
c     body check
      u_np=u(np)
      v_np=v(np)
      w_np=w(np)
      x_np=x(np)
      y_np=y(np)
      z_np=z(np)
c
      do i=1,nbbody(ir_np)
        nozone=ABS(nbzone(i,ir_np))
        jty=itblty(nozone)
        kno=itblno(nozone)
c     rpp check
        if(jty.eq.ityknd(1)) then
          if(kno.le.0.or.kno.gt.irppin) go to 190
          call rppcg1(kno,x_np,y_np,z_np,u_np,v_np,w_np)
c     sph check
        elseif(jty.eq.ityknd(2)) then
          if(kno.le.0.or.kno.gt.isphin) go to 190
          call sphcg1(kno,x_np,y_np,z_np,u_np,v_np,w_np)
c     rcc check
        elseif(jty.eq.ityknd(3)) then
          if(kno.le.0.or.kno.gt.irccin) go to 190
          call rcccg1(kno,x_np,y_np,z_np,u_np,v_np,w_np)
c     trc check
        elseif(jty.eq.ityknd(4)) then
          if(kno.le.0.or.kno.gt.itrcin) go to 190
          call trccg1(kno,x_np,y_np,z_np,u_np,v_np,w_np)
c     tor check
        elseif(jty.eq.ityknd(5)) then
          if(kno.le.0.or.kno.gt.itorin) go to 190
          call torcg1(kno,x_np,y_np,z_np,u_np,v_np,w_np)
c
c**** add new geometry in here
c
       end if
  190  continue
      end do
c
      irnear=ir_np
      if(itvalm.eq.0) then
        tval0=cgeps1
        xidd=x_np+tval0*u_np
        yidd=y_np+tval0*v_np
        zidd=z_np+tval0*w_np
  310   continue
          if(x_np.ne.xidd.or.y_np.ne.yidd.or.z_np.ne.zidd) goto 320
          tval0=tval0*10.d0
          xidd=x_np+tval0*u_np
          yidd=y_np+tval0*v_np
          zidd=z_np+tval0*w_np
          go to 310
  320   continue
c       write(*,*) 'srzone:1'
        call srzone(xidd,yidd,zidd,iq_np,ir_np,irnext)
c
        if(irnext.ne.ir_np) then
          tval=0.0d0
          irnear=irnext
        else
          tval00=0.0d0
          tval10=10.0d0*tval0
          irlold=ir_np
          irlfg=0
  330     continue
          if(irlfg.eq.1) go to 340
            tval00=tval00+tval10
            if(tval00.gt.1.0d+06) then
              write(6,9000) iq(np),ir(np),x(np),y(np),z(np),
     &                      u(np),v(np),w(np),tval00
 9000 format(' TVAL00 ERROR : iq,ir,x,y,z,u,v,w,tval=',
     &       2I3,1P7E12.5)
              stop
            end if
            xidd=x_np+tval00*u_np
            yidd=y_np+tval00*v_np
            zidd=z_np+tval00*w_np
            call srzold(xidd,yidd,zidd,irlold,irlfg)
            go to 330
  340     continue
c
          tval=tval00
          do j=1,10
            xidd=x_np+tval00*u_np
            yidd=y_np+tval00*v_np
            zidd=z_np+tval00*w_np
c           write(*,*) 'srzone:2'
            call srzone(xidd,yidd,zidd,iq_np,irlold,irnext)
            if(irnext.ne.irlold) then
              tval=tval00
              irnear=irnext
            end if
            tval00=tval00-tval0
          end do
          if(ir_np.eq.irnear) then
            write(0,*) 'ir(np),tval=',ir_np,tval
          end if
        end if
      else
        do j=1,itvalm-1
          do i=j+1,itvalm
            if(atval(i).lt.atval(j)) then
              atvaltmp=atval(i)
              atval(i)=atval(j)
              atval(j)=atvaltmp
            endif
          enddo
        enddo
        itvlfg=0
        tvalmn=tval
        do jjj=1,itvalm
          if(tvalmn.gt.atval(jjj)) then
            tvalmn=atval(jjj)
          end if
          delhow=cgeps2
          tval0=atval(jjj)+delhow
          xidd=x_np+tval0*u_np
          yidd=y_np+tval0*v_np
          zidd=z_np+tval0*w_np
  410     continue
          if(x_np.ne.xidd.or.y_np.ne.yidd.or.z_np.ne.zidd) go to 420
            delhow=delhow*10.d0
            tval0=atval(jjj)+delhow
            xidd=x_np+tval0*u_np
            yidd=y_np+tval0*v_np
            zidd=z_np+tval0*w_np
          go to 410
  420     continue
c         write(*,*) 'srzone:3'
          call srzone(xidd,yidd,zidd,iq_np,ir_np,irnext)
          if((irnext.ne.ir_np.or.atval(jjj).ge.1.).and.
     &        tval.gt.atval(jjj)) THEN
            tval=atval(jjj)
            irnear=irnext
            itvlfg=1
            goto 425
          end if
        end do
  425   continue
        if(itvlfg.eq.0) then
          tval0=cgmnst
          xidd=x_np+tval0*u_np
          yidd=y_np+tval0*v_np
          zidd=z_np+tval0*w_np
  430     continue
          if(x_np.ne.xidd.or.y_np.ne.yidd.or.z_np.ne.zidd) go to 440
            tval0=tval0*10.d0
            xidd=x_np+tval0*u_np
            yidd=y_np+tval0*v_np
            zidd=z_np+tval0*w_np
            go to 430
  440     continue
          if(tvalmn.gt.tval0) then
            tval=tvalmn
          else
            tval=tval0
          end if
        end if
      end if
      ihitcg=0
      if(tval.le.ustep) then
        ustep=tval
        ihitcg=1
      end if
      if(ihitcg.eq.1) THEN
        if(irnear.eq.0) THEN
          write(6,9200) iq(np),ir(np),x(np),y(np),z(np),
     &                  u(np),v(np),w(np),tval
 9200 format(' TVAL ERROR : iq,ir,x,y,z,u,v,w,tval=',2I3,1P7E12.5)
          idisc=1
          itverr=itverr+1
          if(itverr.ge.100) then
            stop
          end if
          return
        end if
        irnew=irnear
        if(irnew.ne.ir_np) then
          call rstnxt(iq_np,ir_np,irnew)
        endif
      end if
      return
      end
!--------------------last line of subroutine howfar---------------------
!-------------------------------encoea.f--------------------------------
! Version:   030831-1300
! Reference: SLAC-265 (p.19-20, Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
!
!     real function encoea(energy) 
!     Function to evaluate the energy absorption coefficient of air.    
!      (Tables and Graphs oh photon mass attenuation coefficients and 
!      energy-absorption coefficients for photon energies 1 keV to 
!      20 MeV for elements Z=1 to 92 and some dosimetric materials, 
!      S. M. Seltzer and J. H. Hubbell 1995, Japanese Society of 
!      Radiological Technology)
!----------------------------------------------------------------------
      real function encoea(energy)

      real hnu(38)/0.001,0.0015,0.002,0.003,0.0032029,0.0032029,
     *           0.004,0.005,0.006,0.008,0.01,0.015,0.02,0.03,0.04,
     *           0.05,0.06,0.08,0.10,0.15,0.2,0.3,0.4,0.5,0.6,0.8,1.0,
     *           1.25,1.5,2.0,3.0,4.0,5.0,6.0,8.0,10.0,15.0,20.0/

      real enmu(38)/3599.,  1188.,  526.2,  161.4,  133.0,  146.0,  
     *      76.36, 39.31,  22.70,  9.446,  4.742,  1.334,  0.5389,
     *      0.1537,0.06833,0.04098,0.03041,0.02407,0.02325,0.02496,
     *      0.02672,0.02872,0.02949,0.02966,0.02953,0.02882,0.02789,
     *      0.02666,0.02547,0.02345,0.02057,0.01870,0.01740,0.01647,
     *      0.01525,0.01450,0.01353,0.01311/;
                                                  
      real*8 energy,enm1,hnu1,ene0,slope;

      integer i

      if (energy.gt.hnu(38)) then
        encoea=enmu(38)
        return
      end if
      if (energy.lt.hnu(1)) then
        encoea=enmu(1)
        return
      end if
      
      do i=1,38
        if(energy.ge.hnu(i).and.energy.lt.hnu(i+1)) then
          enm1=alog(enmu(i+1))
          enm0=alog(enmu(i))
          hnu1=alog(hnu(i+1))
          hnu0=alog(hnu(i))

          ene0=dlog(energy)
          slope=(enm1-enm0)/(hnu1-hnu0)
          encoea=exp(enm0+slope*(ene0-hnu0))
          return
        end if
        if(energy.eq.hnu(i+1)) then
          encoea=enmu(i+1)
          return
        end if
      end do

! If sort/interpolation cannot be made, indicate so by writing
! a comment and stopping here.
      write(6,100) energy
100   FORMAT(///,' *****STOPPED IN ENCOEA*****',/,' E=',G15.5,///)
      return
      end
                                                                       
!--------------------------last line of encoea.f------------------------
!-------------------------------encoew.f--------------------------------
! Version:   030831-1300
! Reference: SLAC-265 (p.19-20, Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
!
!     real function encoew(energy) 
!     Function to evaluate the energy absorption coefficient of water.  
!      (Tables and Graphs oh photon mass attenuation coefficients and 
!      energy-absorption coefficients for photon energies 1 keV to 
!      20 MeV for elements Z=1 to 92 and some dosimetric materials, 
!      S. M. Seltzer and J. H. Hubbell 1995, Japanese Society of 
!      Radiological Technology)
!----------------------------------------------------------------------
      real function encoew(energy)

      real hnu(36)/0.001,0.0015,0.002,0.003,0.004,0.005,0.006,0.008,
     *           0.01,0.015,0.02,0.03,0.04,0.05,0.06,0.08,0.10,0.15,
     *           0.2,0.3,0.4,0.5,0.6,0.8,1.0,1.25,1.5,2.0,3.0,4.0,5.0,
     *           6.0,8.0,10.0,15.0,20.0/

      real enmu(36)/4065.,  1372.,  615.2,  191.7,  81.91,  41.88,
     *                24.05,  9.915,  4.944,  1.374,  0.5503, 0.1557,
     *                0.06947,0.04223,0.03190,0.02597,0.02546,0.02764,
     *                0.02967,0.03192,0.03279,0.03299,0.03284,0.03206,
     *                0.03103,0.02965,0.02833,0.02608,0.02281,0.02066,
     *                0.01915,0.01806,0.01658,0.01566,0.01441,0.01382/
                                                  
      real*8 energy,enm1,hnu1,ene0,slope;

      integer i

      if (energy.gt.hnu(36)) then
        encoew=enmu(36)
        return
      end if
      if (energy.lt.hnu(1)) then
        encoew=enmu(1)
        return
      end if
      
      do i=1,36
        if(energy.ge.hnu(i).and.energy.lt.hnu(i+1)) then
          enm1=alog(enmu(i+1))
          enm0=alog(enmu(i))
          hnu1=alog(hnu(i+1))
          hnu0=alog(hnu(i))

          ene0=dlog(energy)
          slope=(enm1-enm0)/(hnu1-hnu0)
          encoew=exp(enm0+slope*(ene0-hnu0))
          return
        end if
        if(energy.eq.hnu(i+1)) then
          encoew=enmu(i+1)
          return
        end if
      end do

! If sort/interpolation cannot be made, indicate so by writing
! a comment and stopping here.
      write(6,100) energy
100   FORMAT(///,' *****STOPPED IN ENCOEW*****',/,' E=',G15.5,///)
      return
      end
!--------------------------last line of encoew.f------------------------
